(***************************************
* TDistractBar 1.2
* Copyright (c) 1999, Rangel Gustavo Reale (hitnrun@geocities.com)
* This code is FREEWARE. Do with it anything you want.
* Use this AT YOUR OWN RISK. I cannot assume anything about it!
* (Altough I dont think there is a risk with a graphics component! ;) )
***************************************)
unit DistBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TDistractBar = class;

  { TDistractBarThread class }

  TDistractBarThread = class(TThread)
  private
    FOwner: TDistractBar;
    FDelay: Integer;
  protected
    procedure DoProcess;
    procedure Execute; override;
  public
    constructor Create(AOwner: TDistractBar);
    property Delay: Integer read FDelay write FDelay;
  end;

  { TDistractBarAnimateProcess class }
  TDistractBarAnimateProcessClass = class of TDistractBarAnimateProcess;

  TDistractBarAnimateProcess = class(TObject)
  private
    FBar: TDistractBar;
  protected
    property Bar: TDistractBar read FBar;
  public
    constructor Create(AOwner: TDistractBar);
    destructor Destroy; override;

    procedure Initialize; virtual;
    procedure Finalize; virtual;

    procedure InitProcess; virtual;
    procedure Process; virtual;
    procedure EndProcess; virtual;
  end;

  { TDistractBar component }
  TDBAnimProcessType = (apNone, apToLeft, apToRight, apBounce, apDouble, apRandom,
     apFillLeft, apFillRight, apFillBounce);
  TDBTextAnimType = (taNone, taToLeft, taToRight, taBounce);
  TDBShowOption = (doDrawLine, doDrawText, doColorOnUseEnd, doColorOffUseEnd,
    doLineUseEnd, doFontUseEnd, doBar3d);
  TDBShowOptions = set of TDBShowOption;
  TDBOrientation = (boHorizontal, boVertical);

  TDistractBar = class(TCustomControl)
  private
    FCaption: string;
    FBarSpacing: Integer;
    FBarCount: Integer;
    FBackgroundColor: TColor;
    FBarColorOn, FBarColorOff, FLineColor: TColor;
    FBarColorOnEnd, FBarColorOffEnd, FLineColorEnd, FFontColorEnd: TColor;
    FEnableMask: Integer;
    FUpdating: Integer;
    FBackScreen: TBitmap;
    FInterval: Integer;
    FAnimated: Boolean;
    FAnimThread: TDistractBarThread;
    FAnimProcess: TDistractBarAnimateProcess;
    FPaintCS, FEnableCS, FProcessCS, FUpdateCS: TRTLCriticalSection;
    FAnimProcessList: array[TDBAnimProcessType] of TDistractBarAnimateProcessClass; 
    FProcess: TDBAnimProcessType;
    FTextSkip: Integer;
    FTextAnim: TDBTextAnimType;
    FAnimateTextLeft: Boolean;
    FOptions: TDBShowOptions;
    FOrientation: TDBOrientation;
    FBarBorderWidth: Integer;
    FOnAnimate: TNotifyEvent;
    FAnimProcessSkip, FAnimTextSkip: Integer;
    FProcessSkipCount, FTextSkipCount: Integer;
    FDestroying: Boolean;

    procedure SetLineColor(const Value: TColor);
    procedure SetCaption(const Value: string);
    procedure SetBarColorOffEnd(const Value: TColor);
    procedure SetBarColorOnEnd(const Value: TColor);
    procedure SetLineColorEnd(const Value: TColor);
    procedure SetTextSkip(const Value: Integer);
    procedure SetTextAnim(const Value: TDBTextAnimType);
    procedure SetFontColorEnd(const Value: TColor);
    procedure SetOptions(const Value: TDBShowOptions);
    procedure SetOrientation(const Value: TDBOrientation);
    procedure SetBarBorderWidth(const Value: Integer);
    procedure SetAnimProcessSkip(const Value: Integer);
    procedure SetAnimTextSkip(const Value: Integer);
  protected
    procedure CheckBackScreen;
    procedure DoAnimate;
    procedure DoAnimateText;
    procedure CheckAnimClass;
    procedure ResetAnim;
    function Interpolate(APercent: Single; AInit, AEnd: Byte): Byte;
    function InterpolateColors(APercent: Single; AColorBegin, AColorEnd: TColor): TColor;

    procedure SetProcess(const Value: TDBAnimProcessType);
    procedure RegisterAnimClasses;
    procedure SetInterval(const Value: Integer);
    procedure SetAnimated(const Value: Boolean);
    function GetBarEnabled(Index: Integer): Boolean;
    procedure SetBarEnabled(Index: Integer; const Value: Boolean);
    procedure SetEnableMask(const Value: Integer);
    procedure SetBackGroundColor(const Value: TColor);
    procedure SetBarColorOn(const Value: TColor);
    procedure SetBarCount(const Value: Integer);
    procedure SetBarColorOff(const Value: TColor);
    procedure SetBarSpacing(const Value: Integer);
    procedure DrawBar(ACanvas: TCanvas; ANumber, ALeft, AWidth: Integer);

    procedure WMEraseBkgnd(var Message: TWmEraseBkgnd); message WM_ERASEBKGND;
    procedure Loaded; override;
    procedure DoPaint;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure BeginUpdate;
    procedure EndUpdate;

    procedure EnableAll;
    procedure DisableAll;

    property BarEnabled[Index: Integer]: Boolean read GetBarEnabled write SetBarEnabled;
  published
    property Align;
    property ShowHint;
    property Font;
    property Animated: Boolean read FAnimated write SetAnimated default False;
    property Caption: string read FCaption write SetCaption;
    property BarSpacing: Integer read FBarSpacing write SetBarSpacing default 3;
    property BarCount: Integer read FBarCount write SetBarCount default 10;
    property BackGroundColor: TColor read FBackGroundColor write SetBackGroundColor default clBtnFace;
    property BarColorOn: TColor read FBarColorOn write SetBarColorOn default clLime;
    property BarColorOff: TColor read FBarColorOff write SetBarColorOff default clGreen;
    property LineColor: TColor read FLineColor write SetLineColor default clBlack;
    property BarColorOnEnd: TColor read FBarColorOnEnd write SetBarColorOnEnd default clLime;
    property BarColorOffEnd: TColor read FBarColorOffEnd write SetBarColorOffEnd default clGreen;
    property LineColorEnd: TColor read FLineColorEnd write SetLineColorEnd default clBlack;
    property FontColorEnd: TColor read FFontColorEnd write SetFontColorEnd default clBlack;
    property EnableMask: Integer read FEnableMask write SetEnableMask default 0;
    property Interval: Integer read FInterval write SetInterval default 100;
    property Process: TDBAnimProcessType read FProcess write SetProcess default apNone;
    property TextSkip: Integer read FTextSkip write SetTextSkip default 0;
    property TextAnim: TDBTextAnimType read FTextAnim write SetTextAnim default taNone;
    property BarBorderWidth: Integer read FBarBorderWidth write SetBarBorderWidth default 1;
    property Options: TDBShowOptions read FOptions write SetOptions default [doDrawLine, doDrawText];
    property Orientation: TDBOrientation read FOrientation write SetOrientation default boHorizontal;
    property AnimProcessSkip: Integer read FAnimProcessSkip write SetAnimProcessSkip default 0;
    property AnimTextSkip: Integer read FAnimTextSkip write SetAnimTextSkip default 0;

    property OnAnimate: TNotifyEvent read FOnAnimate write FOnAnimate;
  end;

  TDBAProcessToLeft = class(TDistractBarAnimateProcess)
  protected
    FSelected: Integer;
  public
    procedure InitProcess; override;
    procedure Process; override;
  end;

  TDBAProcessToRight = class(TDistractBarAnimateProcess)
  protected
    FSelected: Integer;
  public
    procedure InitProcess; override;
    procedure Process; override;
  end;

  TDBAProcessBounce = class(TDistractBarAnimateProcess)
  protected
    FSelected: Integer;
    FGoingLeft: Boolean;
  public
    procedure InitProcess; override;
    procedure Process; override;
  end;

  TDBAProcessDouble = class(TDistractBarAnimateProcess)
  protected
    FSelectedL, FSelectedR: Integer;
  public
    procedure InitProcess; override;
    procedure Process; override;
  end;

  TDBAProcessRandom = class(TDistractBarAnimateProcess)
  public
    procedure InitProcess; override;
    procedure Process; override;
  end;

  TDBAProcessFillLeft = class(TDistractBarAnimateProcess)
  private
    FEnabling: Boolean;
    FSelected: Integer;
  public
    procedure InitProcess; override;
    procedure Process; override;
  end;

  TDBAProcessFillRight = class(TDistractBarAnimateProcess)
  private
    FEnabling: Boolean;
    FSelected: Integer;
  public
    procedure InitProcess; override;
    procedure Process; override;
  end;

  TDBAProcessFillBounce = class(TDistractBarAnimateProcess)
  private
    FEnabling: Boolean;
    FSelected: Integer;
    FGoingLeft: Boolean;
  public
    procedure InitProcess; override;
    procedure Process; override;
  end;

procedure Register;

implementation

function CalcPower(Base, Exponent: Integer): Integer;
var
  I: Integer;
begin
     if Exponent <= 0 then
        Result := 1
     else
     begin
          Result := Base;
          for I := 2 to Exponent do
              Result := Result * Base;
     end;
end;

{ TDistractBarThread }

constructor TDistractBarThread.Create(AOwner: TDistractBar);
begin
     inherited Create(True);
     FOwner := AOwner;
     FDelay := 100;
end;

procedure TDistractBarThread.DoProcess;
begin
     FOwner.DoAnimate;
end;

procedure TDistractBarThread.Execute;
begin
     while not Terminated do
     begin
          //Synchronize(DoProcess);
          DoProcess;
          Sleep(FDelay);
     end;
end;

{ TDistractBarAnimateProcess }

constructor TDistractBarAnimateProcess.Create(AOwner: TDistractBar);
begin
     inherited Create;
     FBar := AOwner;
     Initialize;     
end;

destructor TDistractBarAnimateProcess.Destroy;
begin
     Finalize;
     inherited;
end;

procedure TDistractBarAnimateProcess.EndProcess;
begin
     Bar.DisableAll;
end;

procedure TDistractBarAnimateProcess.Finalize;
begin
     // Do nothing
end;

procedure TDistractBarAnimateProcess.Initialize;
begin
     // Do nothing
end;

procedure TDistractBarAnimateProcess.InitProcess;
begin
     // Do nothing
end;

procedure TDistractBarAnimateProcess.Process;
begin
     // Do noting
end;

{ TDistractBar }

constructor TDistractBar.Create(AOwner: TComponent);
var
  C: TDBAnimProcessType;
begin
     inherited;
     ControlStyle := ControlStyle + [csOpaque];

     InitializeCriticalSection(FPaintCS);
     InitializeCriticalSection(FEnableCS);
     InitializeCriticalSection(FProcessCS);
     InitializeCriticalSection(FUpdateCS);

     FDestroying := False;
     Font.Name := 'Arial';
     Font.Color := clBlack;
     Font.Size := 10;
     FBarSpacing := 3;
     FBarCount := 10;
     FBackgroundColor := clBtnFace;
     FBarColorOn := clLime;
     FBarColorOnEnd := clLime;
     FBarColorOff := clGreen;
     FBarColorOffEnd := clGreen;
     FLineColor := clBlack;
     FLineColorEnd := clBlack;
     FUpdating := 0;
     FAnimated := False;
     FInterval := 100;
     FBackScreen := nil;
     FAnimThread := nil;
     FAnimProcess := nil;
     FTextSkip := 0;
     FAnimateTextLeft := True;
     FOptions := [doDrawLine, doDrawText];
     FOrientation := boHorizontal;
     FBarBorderWidth := 1;
     FAnimProcessSkip := 0;
     FAnimTextSkip := 0;
     FProcessSkipCount := 0;
     FTextSkipCount := 0;

     Width := 150;
     Height := 20;

     for C := Low(TDBAnimProcessType) to High(TDBAnimProcessType) do
         FAnimProcessList[C] := nil;
     RegisterAnimClasses;
end;

destructor TDistractBar.Destroy;
begin
     // Locks drawings
     FDestroying := True;

     //BeginUpdate;
     EnterCriticalSection(FPaintCS);
     try
        SetAnimated(False);
        if FBackScreen <> nil then
        begin
             FBackScreen.Free;
        end;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DeleteCriticalSection(FPaintCS);
     DeleteCriticalSection(FEnableCS);
     DeleteCriticalSection(FProcessCS);
     DeleteCriticalSection(FUpdateCS);
     inherited;
end;

procedure TDistractBar.BeginUpdate;
begin
     EnterCriticalSection(FUpdateCS);
     try
        if FUpdating < 0 then FUpdating := 0;
        Inc(FUpdating);
     finally
        LeaveCriticalSection(FUpdateCS);
     end;
end;

procedure TDistractBar.CheckBackScreen;
begin
     if (FBackScreen = nil) or
        (FBackScreen.Width <> Width) or
        (FBackScreen.Height <> Height) then
     begin
          FBackScreen.Free;
          FBackScreen := nil;

          FBackScreen := TBitmap.Create;
          FBackScreen.Width := Width;
          FBackScreen.Height := Height;          
     end;
end;

procedure TDistractBar.DisableAll;
begin
     EnableMask := 0;
end;

procedure TDistractBar.DoAnimate;
begin
     if FDestroying then Exit;
     if not FAnimated then Exit;

     BeginUpdate;
     try
        EnterCriticalSection(FProcessCS);
        try
           Inc(FProcessSkipCount);
           if FProcessSkipCount > FAnimProcessSkip then
           begin
                if Assigned(FAnimProcess) then
                begin
                     FAnimProcess.Process;
                end;
                FProcessSkipCount := 0;
           end;

           Inc(FTextSkipCount);
           if FTextSkipCount > FAnimTextSkip then
           begin
                if FTextAnim <> taNone then
                   DoAnimateText;
                FTextSkipCount := 0;
           end;
        finally
           LeaveCriticalSection(FProcessCS);
        end;
     finally
        EndUpdate;
     end;

     if Assigned(FOnAnimate) then
        FOnAnimate(Self);
end;

procedure TDistractBar.DrawBar(ACanvas: TCanvas; ANumber, ALeft, AWidth: Integer);
var
  R: TRect;
  C: TColor;
  S: string;
  AChar: SmallInt;
begin
     case FOrientation of
       boVertical: R := Rect(0, ALeft, Width, ALeft+AWidth);
     else
       R := Rect(ALeft, 0, ALeft+AWidth, Height);
     end;

     if doBar3d in FOptions then
     begin
          R.Right := R.Right-FBarBorderWidth;
          R.Bottom := R.Bottom-FBarBorderWidth;
     end;

     if GetBarEnabled(ANumber) then
     begin
          if (doColorOnUseEnd in FOptions) and
             (FBarColorOn <> FBarColorOnEnd) then
          begin
               C := InterpolateColors((ANumber+1) / FBarCount, FBarColorOn, FBarColorOnEnd);
          end
          else
              C := FBarColorOn;
     end
     else
     begin
          if (doColorOffUseEnd in FOptions) and
             (FBarColorOff <> FBarColorOffEnd) then
          begin
               C := InterpolateColors((ANumber+1) / FBarCount, FBarColorOff, FBarColorOffEnd);
          end
          else
              C := FBarColorOff;
     end;

     ACanvas.Brush.Color := C;
     ACanvas.FillRect(R);

     if doDrawLine in FOptions then
     begin
          if (doLineUseEnd in FOptions) and
             (FLineColor <> FLineColorEnd) then
             C := InterpolateColors((ANumber+1) / FBarCount, FLineColor, FLineColorEnd)
          else
              C := FLineColor;

          ACanvas.Brush.Color := C;
          ACanvas.FrameRect(R);
     end;

     if doDrawText in FOptions then
     begin
          ACanvas.Font.Assign(Font);
          if (doFontUseEnd in FOptions) and
             (Font.Color <> FFontColorEnd) then
             ACanvas.Font.Color := InterpolateColors((ANumber+1) / FBarCount, Font.Color, FFontColorEnd);

          AChar := ANumber + 1 - FTextSkip;
          if AChar <= 0 then AChar := FBarCount + AChar;
          if (AChar > 0) and (Length(FCaption) >= AChar) then
          begin
               S := FCaption[AChar];
               SetBkMode(ACanvas.Handle, TRANSPARENT);
               DrawTextEx(ACanvas.Handle, PChar(S), -1, R,
                  DT_CENTER or DT_SINGLELINE or DT_VCENTER, nil);
          end;
     end;

     if doBar3d in FOptions then
     begin
          ACanvas.Brush.Color := clBlack;
          ACanvas.FillRect(Rect(R.Left+FBarBorderWidth, R.Bottom,
               R.Right+FBarBorderWidth, R.Bottom+FBarBorderWidth));
          ACanvas.FillRect(Rect(R.Right, R.Top+FBarBorderWidth,
               R.Right+FBarBorderWidth, R.Bottom+FBarBorderWidth));
     end;
end;

procedure TDistractBar.EnableAll;
begin
     EnableMask := CalcPower(2, FBarCount);
end;

procedure TDistractBar.EndUpdate;
begin
     EnterCriticalSection(FUpdateCS);
     try
        if FUpdating > 0 then
        begin
             Dec(FUpdating);
             if FUpdating = 0 then
                DoPaint;
        end;
     finally
        LeaveCriticalSection(FUpdateCS);
     end;
end;

function TDistractBar.GetBarEnabled(Index: Integer): Boolean;
var
  N: Integer;
begin
     if (Index < 0) or (Index > FBarCount-1) then
     begin
          Result := False;
          Exit;
     end;
     N := CalcPower(2, Index);
     EnterCriticalSection(FEnableCS);
     try
        Result := FEnableMask and N = N;
     finally
        LeaveCriticalSection(FEnableCS);
     end;
end;

procedure TDistractBar.Paint;
var
  BW, I, M: Integer;
  Margin: Integer;
begin
     if FDestroying then Exit;
     if FUpdating <> 0 then Exit;

     EnterCriticalSection(FPaintCS);
     try
        CheckBackScreen;

        case FOrientation of
          boVertical: M := Height;
        else
          M := Width;
        end;

        // Width of bars
        BW := (M - ((FBarCount - 1) * FBarSpacing)) div FBarCount;
        Margin := (M - (BW * FBarCount) - (FBarSpacing * (FBarCount-1))) div 2; 

        FBackScreen.Canvas.Brush.Color := FBackgroundColor;
        FBackScreen.Canvas.FillRect(Rect(0, 0, Width, Height));

        for I := 0 to FBarCount-1 do
        begin
             DrawBar(FBackScreen.Canvas, I, Margin + (BW * I) + (FBarSpacing * I), BW);
        end;

        // Draw the buffer onscreen
        // Sometimes Delphi show a "Control XXX has no parent window" when destroying.
        // Doing this seen to fix it
        if (Parent <> nil) then
           Canvas.Draw(0, 0, FBackScreen);
     finally
        LeaveCriticalSection(FPaintCS);
     end;
end;

procedure TDistractBar.SetAnimated(const Value: Boolean);
begin
     if Value <> FAnimated then
     begin
          FAnimated := Value;

          if Value then
          begin
               CheckAnimClass;

               FAnimThread := TDistractBarThread.Create(Self);
               FAnimThread.Delay := FInterval;
               if (not (csLoading in ComponentState)) and
                  (not (csReading in ComponentState)) then
                  FAnimThread.Resume;
          end
          else
          begin
               if FAnimThread <> nil then
               begin
                    FAnimThread.Terminate;
                    FAnimThread.WaitFor;

                    FAnimThread.Free;
                    FAnimThread := nil;

                    CheckAnimClass;
               end;
          end;
     end;
end;

procedure TDistractBar.SetBackGroundColor(const Value: TColor);
begin
     EnterCriticalSection(FPaintCS);
     try
        FBackGroundColor := Value;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetBarColorOff(const Value: TColor);
begin
     EnterCriticalSection(FPaintCS);
     try
        FBarColorOff := Value;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetBarColorOn(const Value: TColor);
begin
     EnterCriticalSection(FPaintCS);
     try
        FBarColorOn := Value;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetLineColor(const Value: TColor);
begin
     EnterCriticalSection(FPaintCS);
     try
        FLineColor := Value;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetBarColorOffEnd(const Value: TColor);
begin
     EnterCriticalSection(FPaintCS);
     try
        FBarColorOffEnd := Value;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetBarColorOnEnd(const Value: TColor);
begin
     EnterCriticalSection(FPaintCS);
     try
        FBarColorOnEnd := Value;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetLineColorEnd(const Value: TColor);
begin
     EnterCriticalSection(FPaintCS);
     try
        FLineColorEnd := Value;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetFontColorEnd(const Value: TColor);
begin
     EnterCriticalSection(FPaintCS);
     try
        FFontColorEnd := Value;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetOptions(const Value: TDBShowOptions);
begin
     EnterCriticalSection(FPaintCS);
     try
        FOptions := Value;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetOrientation(const Value: TDBOrientation);
begin
     EnterCriticalSection(FPaintCS);
     try
        FOrientation := Value;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetBarBorderWidth(const Value: Integer);
begin
     if Value > 0 then
     begin
          EnterCriticalSection(FPaintCS);
          try
             FBarBorderWidth := Value;
          finally
             LeaveCriticalSection(FPaintCS);
          end;
          DoPaint;
     end
     else
         raise Exception.Create('Invalid width');
end;

procedure TDistractBar.SetAnimProcessSkip(const Value: Integer);
begin
     if Value >= 0 then
     begin
          EnterCriticalSection(FProcessCS);
          try
             FAnimProcessSkip := Value;
          finally
             LeaveCriticalSection(FProcessCS);
          end;
     end
     else
         raise Exception.Create('Invalid value');
end;

procedure TDistractBar.SetAnimTextSkip(const Value: Integer);
begin
     if Value >= 0 then
     begin
          EnterCriticalSection(FProcessCS);
          try
             FAnimTextSkip := Value;
          finally
             LeaveCriticalSection(FProcessCS);
          end;
     end
     else
         raise Exception.Create('Invalid value');
end;

procedure TDistractBar.SetCaption(const Value: string);
begin
     EnterCriticalSection(FPaintCS);
     try
        FCaption := Value;
     finally
        LeaveCriticalSection(FPaintCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetTextSkip(const Value: Integer);
begin
     if Value >= 0 then
     begin
          EnterCriticalSection(FPaintCS);
          try
             FTextSkip := Value;
          finally
             LeaveCriticalSection(FPaintCS);
          end;
          DoPaint;
     end
     else
         raise Exception.Create('Invalid value');
end;

procedure TDistractBar.SetBarCount(const Value: Integer);
begin
     if Value > 0 then
     begin
          EnterCriticalSection(FPaintCS);
          try
             FBarCount := Value;
          finally
             LeaveCriticalSection(FPaintCS);
          end;
          ResetAnim;
          DoPaint;
     end
     else
         raise Exception.Create('Invalid count');
end;

procedure TDistractBar.SetBarEnabled(Index: Integer; const Value: Boolean);
var
  N: Integer;
begin
     if (Index < 0) or (Index > FBarCount-1) then Exit;

     EnterCriticalSection(FEnableCS);
     try
        N := CalcPower(2, Index);
        if Value then
           FEnableMask := FEnableMask or N
        else
           FEnableMask := FEnableMask and (not N);
     finally
        LeaveCriticalSection(FEnableCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetBarSpacing(const Value: Integer);
begin
     if Value >= 0 then
     begin
          EnterCriticalSection(FPaintCS);
          try
             FBarSpacing := Value;
          finally
             LeaveCriticalSection(FPaintCS);
          end;
          DoPaint;
     end
     else
         raise Exception.Create('Invalid width')
end;

procedure TDistractBar.SetEnableMask(const Value: Integer);
begin
     EnterCriticalSection(FEnableCS);
     try
        FEnableMask := Value;
     finally
        LeaveCriticalSection(FEnableCS);
     end;
     DoPaint;
end;

procedure TDistractBar.SetInterval(const Value: Integer);
begin
     FInterval := Value;
     if (FAnimated) and
        (FAnimThread <> nil) then
        FAnimThread.Delay := Value;
end;

procedure TDistractBar.Loaded;
begin
     inherited;
     if (FAnimated) and
        (FAnimThread <> nil) then
        FAnimThread.Resume;
end;

procedure TDistractBar.RegisterAnimClasses;
begin
     FAnimProcessList[apToLeft] := TDBAProcessToLeft;
     FAnimProcessList[apToRight] := TDBAProcessToRight;
     FAnimProcessList[apBounce] := TDBAProcessBounce;
     FAnimProcessList[apDouble] := TDBAProcessDouble;
     FAnimProcessList[apRandom] := TDBAProcessRandom;
     FAnimProcessList[apFillLeft] := TDBAProcessFillLeft;
     FAnimProcessList[apFillRight] := TDBAProcessFillRight;
     FAnimProcessList[apFillBounce] := TDBAProcessFillBounce;
end;

procedure TDistractBar.SetProcess(const Value: TDBAnimProcessType);
begin
     if Value <> FProcess then
     begin
          FProcess := Value;
          CheckAnimClass;
     end;
end;

procedure TDistractBar.SetTextAnim(const Value: TDBTextAnimType);
begin
     if Value <> FTextAnim then
     begin
          EnterCriticalSection(FProcessCS);
          try
             FTextAnim := Value;
             FAnimateTextLeft := True;
             FTextSkip := 0;
          finally
             LeaveCriticalSection(FProcessCS);
          end;
     end;
end;

procedure TDistractBar.CheckAnimClass;
begin
     EnterCriticalSection(FProcessCS);
     try
        if FAnimProcess <> nil then
        begin
             FAnimProcess.EndProcess;
             FAnimProcess.Free;
             FAnimProcess := nil;
        end;

        FProcessSkipCount := 0;
        FTextSkipCount := 0;

        if (FAnimated) and
           (FProcess <> apNone) and
           (FAnimProcessList[FProcess] <> nil) then
        begin
             FAnimProcess := FAnimProcessList[FProcess].Create(Self);
             FAnimProcess.InitProcess;
        end;
     finally
        LeaveCriticalSection(FProcessCS);
     end;
end;


procedure TDistractBar.ResetAnim;
begin
     if FAnimProcess <> nil then
     begin
          EnterCriticalSection(FProcessCS);
          try
             FAnimProcess.InitProcess;
          finally
             LeaveCriticalSection(FProcessCS);
          end;
     end;
end;

function TDistractBar.Interpolate(APercent: Single; AInit, AEnd: Byte): Byte;
var
  Dist: SmallInt;
begin
     Dist := Trunc(Abs(AInit-AEnd) * APercent);
     if AInit < AEnd then
        Result := AInit + Dist
     else
         Result := AInit - Dist;
end;

function TDistractBar.InterpolateColors(APercent: Single; AColorBegin,
  AColorEnd: TColor): TColor;
begin
     Result := RGB(Interpolate(APercent, GetRValue(AColorBegin), GetRValue(AColorEnd)),
                   Interpolate(APercent, GetGValue(AColorBegin), GetGValue(AColorEnd)),
                   Interpolate(APercent, GetBValue(AColorBegin), GetBValue(AColorEnd)));
end;

procedure TDistractBar.DoAnimateText;
begin
     case FTextAnim of
       taToLeft:
         begin
              FTextSkip := FTextSkip + 1;
              if FTextSkip > FBarCount-1 then
                 FTextSkip := 0;
         end;
       taToRight:
         begin
              FTextSkip := FTextSkip - 1;
              if FTextSkip < 0 then
                 FTextSkip := FBarCount-1;
         end;
       taBounce:
         begin
              if FAnimateTextLeft then
              begin
                   FTextSkip := FTextSkip + 1;
                   if FTextSkip+Length(FCaption) > FBarCount then
                   begin
                        FTextSkip := FBarCount-Length(FCaption)-1;
                        FAnimateTextLeft := False;
                   end;
              end
              else
              begin
                   FTextSkip := FTextSkip - 1;
                   if FTextSkip < 0 then
                   begin
                        FTextSkip := 1;
                        FAnimateTextLeft := True;
                   end;
              end;
         end;
     end;
end;

procedure TDistractBar.WMEraseBkgnd(var Message: TWmEraseBkgnd);
begin
     Message.Result := 1;
end;

procedure TDistractBar.DoPaint;
begin
     Canvas.Lock;
     try
        Paint;
     finally
        Canvas.Unlock;
     end;
end;

{ TDBAProcessToLeft }

procedure TDBAProcessToLeft.InitProcess;
begin
     inherited;
     FSelected := 0;
end;

procedure TDBAProcessToLeft.Process;
begin
     Bar.BeginUpdate;
     try
        if FSelected > Bar.BarCount-1 then
           FSelected := 0;

        Bar.DisableAll;
        Bar.BarEnabled[FSelected] := True;

        Inc(FSelected);
     finally
        Bar.EndUpdate;
     end;
end;

{ TDBAProcessToRight }

procedure TDBAProcessToRight.InitProcess;
begin
     inherited;
     FSelected := Bar.BarCount-1;
end;

procedure TDBAProcessToRight.Process;
begin
     Bar.BeginUpdate;
     try
        if FSelected < 0 then
           FSelected := Bar.BarCount;

        Bar.DisableAll;
        Bar.BarEnabled[FSelected] := True;

        Dec(FSelected);
     finally
        Bar.EndUpdate;
     end;
end;

{ TDBAProcessBounce }

procedure TDBAProcessBounce.InitProcess;
begin
     inherited;
     FSelected := 0;
     FGoingLeft := True;
end;

procedure TDBAProcessBounce.Process;
begin
     Bar.BeginUpdate;
     try
        if FGoingLeft then
        begin
             if FSelected > Bar.BarCount-1 then
             begin
                  FSelected := Bar.BarCount-2;
                  FGoingLeft := False;
             end;
        end
        else
        begin
             if FSelected < 0 then
             begin
                  FSelected := 1;
                  FGoingLeft := True;
             end;
        end;

        Bar.DisableAll;
        Bar.BarEnabled[FSelected] := True;

        if FGoingLeft then
           Inc(FSelected)
        else
            Dec(FSelected);
     finally
        Bar.EndUpdate;
     end;
end;

{ TDBAProcessDouble }

procedure TDBAProcessDouble.InitProcess;
begin
     inherited;
     FSelectedL := 0;
     FSelectedR := Bar.BarCount-1;
end;

procedure TDBAProcessDouble.Process;
begin
     Bar.BeginUpdate;
     try
        if FSelectedL > Bar.BarCount-1 then
        begin
             FSelectedL := 0;
        end;

        if FSelectedR < 0 then
        begin
             FSelectedR := Bar.BarCount-1;
        end;

        Bar.DisableAll;
        Bar.BarEnabled[FSelectedL] := True;
        Bar.BarEnabled[FSelectedR] := True;

        Inc(FSelectedL);
        Dec(FSelectedR);
     finally
        Bar.EndUpdate;
     end;
end;

procedure Register;
begin
  RegisterComponents('Dialogs', [TDistractBar]);
end;


{ TDBAProcessRandom }

procedure TDBAProcessRandom.InitProcess;
begin
     Randomize;
end;

procedure TDBAProcessRandom.Process;
begin
     Bar.BeginUpdate;
     try
        Bar.DisableAll;

        Bar.EnableMask := Random(CalcPower(2, Bar.BarCount));
     finally
        Bar.EndUpdate;
     end;
end;

{ TDBAProcessFillLeft }

procedure TDBAProcessFillLeft.InitProcess;
begin
     inherited;
     FEnabling := True;
     FSelected := 0;
     Bar.DisableAll;
end;

procedure TDBAProcessFillLeft.Process;
begin
     inherited;
     Bar.BeginUpdate;
     try
        Bar.BarEnabled[FSelected] := FEnabling;
        Inc(FSelected);
        if FSelected > Bar.BarCount-1 then
        begin
             FSelected := 0;
             FEnabling := not FEnabling;
        end;
     finally
        Bar.EndUpdate;
     end;
end;

{ TDBAProcessFillRight }

procedure TDBAProcessFillRight.InitProcess;
begin
     inherited;
     FEnabling := True;
     FSelected := Bar.BarCount-1;
     Bar.DisableAll;
end;

procedure TDBAProcessFillRight.Process;
begin
     inherited;
     Bar.BeginUpdate;
     try
        Bar.BarEnabled[FSelected] := FEnabling;
        Dec(FSelected);
        if FSelected < 0 then
        begin
             FSelected := Bar.BarCount-1;
             FEnabling := not FEnabling;
        end;
     finally
        Bar.EndUpdate;
     end;
end;

{ TDBAProcessFillBounce }

procedure TDBAProcessFillBounce.InitProcess;
begin
     inherited;
     FEnabling := True;
     FSelected := 0;
     FGoingLeft := True;
     Bar.DisableAll;
end;

procedure TDBAProcessFillBounce.Process;
begin
     inherited;
     Bar.BeginUpdate;
     try
        Bar.BarEnabled[FSelected] := FEnabling;
        if FGoingLeft then
        begin
             Inc(FSelected);
             if FSelected > Bar.BarCount-1 then
             begin
                  if FEnabling then
                  begin
                       FSelected := Bar.BarCount-1;
                       FGoingLeft := False;
                  end
                  else
                  begin
                       FSelected := 0;
                       FGoingLeft := True;
                  end;
                  FEnabling := not FEnabling;
             end;
        end
        else
        begin
             Dec(FSelected);
             if FSelected < 0 then
             begin
                  if FEnabling then
                  begin
                       FSelected := 0;
                       FGoingLeft := True;
                  end
                  else
                  begin
                       FSelected := Bar.BarCount-1;
                       FGoingLeft := False;
                  end;
                  FEnabling := not FEnabling;
             end;
        end;
     finally
        Bar.EndUpdate;
     end;
end;

end.
